Attribute VB_Name = "modLL_UTM"
'============================================================
' Descrizione.....: Conversione di Latitudine/Longitudine
'                   da/a UTM (Universal Transverse Mercator).
' Nome dei Files..: modLL_UTM
' Data............: 29/8/2000
' Versione........: 1.0 a 32 bits.
'                   (Volpian 2.1)
' Sistema.........: VB6 sotto Windows NT.
' Scritto da......: F. Languasco 
' E-Mail..........: MC7061@mclink.it
'============================================================
'
'   Ringrazio Fabio Guerrazzi (fabiog@si.tdnet.it) per
'   la segnalazione del sito:
'       http://www.gpsy.com/gpsinfo/geotoutm/
'   dove ho trovato queste routines, a lungo cercate.
'
'   Routines tradotte dal C.
'   Written by Chuck Gantz - chuck.gantz@globalstar.com
'
Option Explicit
'
Private Type Ellipsoid_Type
    Name As String
    EquatorialRadius As Double
    EccentricitySquared As Double
End Type
Private EllipsoidT(1 To 23) As Ellipsoid_Type
'
Private Const PI# = 3.14159265358979
Private Const FOURTHPI# = PI / 4#
Private Const deg2rad# = PI / 180#
Private Const rad2deg# = 180# / PI
Private Const k0# = 0.9996
Public Sub LLtoUTM(ByVal ReferenceEllipsoid&, ByVal Lat#, ByVal Lng# _
    , UTMNorthing#, UTMEasting#, UTMZone$)
'
'   Converts lat/long to UTM coords.  Equations from USGS Bulletin 1532.
'   East Longitudes are positive, West longitudes are negative.
'   North latitudes are positive, South latitudes are negative.
'   Lat and Lng are in decimal degrees.
'   Written in C by Chuck Gantz - chuck.gantz@globalstar.com
'
'   Source
'   Defense Mapping Agency. 1987b. DMA Technical Report:
'   Supplement to Department of Defense World Geodetic System.
'   1984 Technical Report. Part I and II. Washington, DC: Defense Mapping Agency
'
    Dim eccSquared#
    Dim LngOrigin#, eccPrimeSquared#
    Dim ae#, A#, N#, T#, C#, M#
    Dim LngTemp#, LatRad#, LngRad#, LngOriginRad#
    Dim ZoneNumber&
'
    ' Prepara, se e' il caso, la tavola EllipsoidT():
    If EllipsoidT(1).Name = "" Then Ellipsoid
'
    ae = EllipsoidT(ReferenceEllipsoid).EquatorialRadius
    eccSquared = EllipsoidT(ReferenceEllipsoid).EccentricitySquared
    ' Make sure the longitude is between -180.00 .. 179.9:
    LngTemp = (Lng + 180) - Int((Lng + 180) / 360) * 360 - 180 ' -180.00 .. 179.9;
'
    LatRad = Lat * deg2rad
    LngRad = LngTemp * deg2rad
    ZoneNumber = Int((LngTemp + 180) / 6) + 1
'
    If (Lat >= 56# And Lat < 64# And LngTemp >= 3# And LngTemp < 12#) Then
        ZoneNumber = 32
    End If
'
    ' Special zones for Svalbard:
    If (Lat >= 72# And Lat < 84#) Then
        If (LngTemp >= 0# And LngTemp < 9#) Then
            ZoneNumber = 31
        ElseIf (LngTemp >= 9# And LngTemp < 21#) Then
            ZoneNumber = 33
        ElseIf (LngTemp >= 21# And LngTemp < 33#) Then
            ZoneNumber = 35
        ElseIf (LngTemp >= 33# And LngTemp < 42#) Then
            ZoneNumber = 37
        End If
    End If
    LngOrigin = (ZoneNumber - 1) * 6 - 180 + 3 ' +3 puts origin in middle of zone.
    LngOriginRad = LngOrigin * deg2rad
'
    ' Compute the UTM Zone from the latitude and longitude:
    UTMZone$ = ZoneNumber & " " & UTMLetterDesignator(Lat)
'
    eccPrimeSquared = eccSquared / (1# - eccSquared)
'
    N = ae / Sqr(1# - eccSquared * Sin(LatRad) * Sin(LatRad))
    T = Tan(LatRad) * Tan(LatRad)
    C = eccPrimeSquared * Cos(LatRad) * Cos(LatRad)
    A = Cos(LatRad) * (LngRad - LngOriginRad)
'
    M = ae * ((1 - eccSquared / 4 - 3 * eccSquared * eccSquared / 64 _
      - 5 * eccSquared * eccSquared * eccSquared / 256) * LatRad _
      - (3 * eccSquared / 8 + 3 * eccSquared * eccSquared / 32 _
      + 45 * eccSquared * eccSquared * eccSquared / 1024) * Sin(2 * LatRad) _
      + (15 * eccSquared * eccSquared / 256 _
      + 45 * eccSquared * eccSquared * eccSquared / 1024) * Sin(4 * LatRad) _
      - (35 * eccSquared * eccSquared * eccSquared / 3072) * Sin(6 * LatRad))
'
    UTMEasting = (k0 * N * (A + (1 - T + C) * A * A * A / 6 _
               + (5 - 18 * T + T * T + 72 * C _
               - 58 * eccPrimeSquared) * A * A * A * A * A / 120) _
               + 500000#)
'
    UTMNorthing = (k0 * (M + N * Tan(LatRad) * (A * A / 2 _
                + (5 - T + 9 * C + 4 * C * C) * A * A * A * A / 24 _
                + (61 - 58 * T + T * T + 600 * C _
                - 330 * eccPrimeSquared) * A * A * A * A * A * A / 720)))
    If (Lat < 0) Then
        ' 10000000 meter offset for southern hemisphere:
        UTMNorthing = UTMNorthing + 10000000#
    End If
'
'
'
End Sub

Public Sub UTMtoLL(ByVal ReferenceEllipsoid&, ByVal UTMNorthing#, ByVal UTMEasting# _
    , ByVal UTMZone$, Lat#, Lng#)
'
'   Converts UTM coords to Lat/Lng.  Equations from USGS Bulletin 1532.
'   East Longitudes are positive, West longitudes are negative.
'   North latitudes are positive, South latitudes are negative.
'   Lat and Lng are in decimal degrees.
'   Written in C by Chuck Gantz - chuck.gantz@globalstar.com
'
    Dim ae#, e1#, eccSquared#, eccPrimeSquared#
    Dim N1#, T1#, C1#, R1#, D#, M#, X#, Y#
    Dim LngOrigin#, mu#, phi1#, phi1Rad#
    Dim ZoneNumber&, ZoneLetter$
    Dim NorthernHemisphere&  ' 1 for northern hemispher, 0 for southern.
'
    ' Prepara, se e' il caso, la tavola EllipsoidT():
    If EllipsoidT(1).Name = "" Then Ellipsoid
'
    ae = EllipsoidT(ReferenceEllipsoid).EquatorialRadius
    eccSquared = EllipsoidT(ReferenceEllipsoid).EccentricitySquared
    e1 = (1# - Sqr(1# - eccSquared)) / (1# + Sqr(1# - eccSquared))
'
    X = UTMEasting - 500000# ' Remove 500,000 meter offset for longitude.
    Y = UTMNorthing
'
    ZoneNumber = Val(UTMZone$)
    ZoneLetter$ = UCase$(Trim$(Mid$(UTMZone$, InStr(UTMZone$, " "))))
    If ZoneLetter$ >= "N" Then
        NorthernHemisphere = 1  ' Point is in northern hemisphere.
    Else
        NorthernHemisphere = 0  ' Point is in southern hemisphere.
        Y = Y - 10000000#       ' Remove 10,000,000 meter offset used
                                ' for southern hemisphere.
    End If
'
    LngOrigin = CDbl(ZoneNumber - 1) * 6# - 180# + 3# ' +3 puts origin in middle of zone.
'
    eccPrimeSquared = (eccSquared) / (1# - eccSquared)
'
    M = Y / k0
    mu = M / (ae * (1# - eccSquared / 4# - 3# * eccSquared * eccSquared / 64# _
       - 5# * eccSquared * eccSquared * eccSquared / 256#))
'
    phi1Rad = mu + (3# * e1 / 2# - 27# * e1 * e1 * e1 / 32#) * Sin(2# * mu) _
            + (21# * e1 * e1 / 16# - 55# * e1 * e1 * e1 * e1 / 32#) * Sin(4# * mu) _
            + (151# * e1 * e1 * e1 / 96#) * Sin(6# * mu)
    phi1 = phi1Rad * rad2deg
'
    N1 = ae / Sqr(1# - eccSquared * Sin(phi1Rad) * Sin(phi1Rad))
    T1 = Tan(phi1Rad) * Tan(phi1Rad)
    C1 = eccPrimeSquared * Cos(phi1Rad) * Cos(phi1Rad)
    R1 = ae * (1# - eccSquared) / ((1# - eccSquared _
       * (Sin(phi1Rad) * Sin(phi1Rad)) ^ 1.5))
    D = X / (N1 * k0)
'
    Lat = phi1Rad - (N1 * Tan(phi1Rad) / R1) * (D * D / 2# _
        - (5# + 3# * T1 + 10 * C1 - 4# * C1 * C1 _
        - 9# * eccPrimeSquared) * D * D * D * D / 24# _
        + (61# + 90# * T1 + 298 * C1 + 45# * T1 * T1 _
        - 252# * eccPrimeSquared - 3# * C1 * C1) * D * D * D * D * D * D / 720#)
    Lat = Lat * rad2deg
'
    Lng = (D - (1# + 2# * T1 + C1) * D * D * D / 6# _
        + (5# - 2# * C1 + 28 * T1 - 3# * C1 * C1 _
        + 8# * eccPrimeSquared + 24# * T1 * T1) _
        * D * D * D * D * D / 120#) / Cos(phi1Rad)
    Lng = LngOrigin + Lng * rad2deg
'
'
'
End Sub

Private Sub Ellipsoid()
'
'   Reference ellipsoids derived from Peter H. Dana's website:
'   http://www.utexas.edu/depts/grg/gcraft/notes/datum/elist.html
'   Department of Geography, University of Texas at Austin
'   Internet: pdana@mail.utexas.edu
'   3/22/95
'
    EllipsoidT(1).Name = "Airy"
    EllipsoidT(1).EquatorialRadius = 6377563
    EllipsoidT(1).EccentricitySquared = 0.00667054
'
    EllipsoidT(2).Name = "Australian National"
    EllipsoidT(2).EquatorialRadius = 6378160
    EllipsoidT(2).EccentricitySquared = 0.006694542
'
    EllipsoidT(3).Name = "Bessel 1841"
    EllipsoidT(3).EquatorialRadius = 6377397
    EllipsoidT(3).EccentricitySquared = 0.006674372
'
    EllipsoidT(4).Name = "Bessel 1841 (Nambia) "
    EllipsoidT(4).EquatorialRadius = 6377484
    EllipsoidT(4).EccentricitySquared = 0.006674372
'
    EllipsoidT(5).Name = "Clarke 1866"
    EllipsoidT(5).EquatorialRadius = 6378206
    EllipsoidT(5).EccentricitySquared = 0.006768658
'
    EllipsoidT(6).Name = "Clarke 1880"
    EllipsoidT(6).EquatorialRadius = 6378249
    EllipsoidT(6).EccentricitySquared = 0.006803511
'
    EllipsoidT(7).Name = "Everest"
    EllipsoidT(7).EquatorialRadius = 6377276
    EllipsoidT(7).EccentricitySquared = 0.006637847
'
    EllipsoidT(8).Name = "Fischer 1960 (Mercury) "
    EllipsoidT(8).EquatorialRadius = 6378166
    EllipsoidT(8).EccentricitySquared = 0.006693422
'
    EllipsoidT(9).Name = "Fischer 1968"
    EllipsoidT(9).EquatorialRadius = 6378150
    EllipsoidT(9).EccentricitySquared = 0.006693422
'
    EllipsoidT(10).Name = "GRS 1967"
    EllipsoidT(10).EquatorialRadius = 6378160
    EllipsoidT(10).EccentricitySquared = 0.006694605
'
    EllipsoidT(11).Name = "GRS 1980"
    EllipsoidT(11).EquatorialRadius = 6378137
    EllipsoidT(11).EccentricitySquared = 0.00669438
'
    EllipsoidT(12).Name = "Helmert 1906"
    EllipsoidT(12).EquatorialRadius = 6378200
    EllipsoidT(12).EccentricitySquared = 0.006693422
'
    EllipsoidT(13).Name = "Hough"
    EllipsoidT(13).EquatorialRadius = 6378270
    EllipsoidT(13).EccentricitySquared = 0.00672267
'
    EllipsoidT(14).Name = "International"
    EllipsoidT(14).EquatorialRadius = 6378388
    EllipsoidT(14).EccentricitySquared = 0.00672267
'
    EllipsoidT(15).Name = "Krassovsky"
    EllipsoidT(15).EquatorialRadius = 6378245
    EllipsoidT(15).EccentricitySquared = 0.006693422
'
    EllipsoidT(16).Name = "Modified Airy"
    EllipsoidT(16).EquatorialRadius = 6377340
    EllipsoidT(16).EccentricitySquared = 0.00667054
'
    EllipsoidT(17).Name = "Modified Everest"
    EllipsoidT(17).EquatorialRadius = 6377304
    EllipsoidT(17).EccentricitySquared = 0.006637847
'
    EllipsoidT(18).Name = "Modified Fischer 1960"
    EllipsoidT(18).EquatorialRadius = 6378155
    EllipsoidT(18).EccentricitySquared = 0.006693422
'
    EllipsoidT(19).Name = "South American 1969"
    EllipsoidT(19).EquatorialRadius = 6378160
    EllipsoidT(19).EccentricitySquared = 0.006694542
'
    EllipsoidT(20).Name = "WGS 60"
    EllipsoidT(20).EquatorialRadius = 6378165
    EllipsoidT(20).EccentricitySquared = 0.006693422
'
    EllipsoidT(21).Name = "WGS 66"
    EllipsoidT(21).EquatorialRadius = 6378145
    EllipsoidT(21).EccentricitySquared = 0.006694542
'
    EllipsoidT(22).Name = "WGS-72"
    EllipsoidT(22).EquatorialRadius = 6378135
    EllipsoidT(22).EccentricitySquared = 0.006694318
'
    EllipsoidT(23).Name = "WGS-84"
    EllipsoidT(23).EquatorialRadius = 6378137
    EllipsoidT(23).EccentricitySquared = 0.00669438
'
'
'
End Sub

Private Function UTMLetterDesignator(ByVal Lat#) As String
'
'   This routine determines the correct UTM letter designator for the
'   given latitude.
'   Returns "Z" if latitude is outside the UTM limits of 84N to 80S.
'   Written by Chuck Gantz - chuck.gantz@globalstar.com
'
    If ((84# >= Lat) And (Lat >= 72#)) Then
        UTMLetterDesignator = "X"
    ElseIf ((72# > Lat) And (Lat >= 64#)) Then
        UTMLetterDesignator = "W"
    ElseIf ((64# > Lat) And (Lat >= 56#)) Then
        UTMLetterDesignator = "V"
    ElseIf ((56# > Lat) And (Lat >= 48#)) Then
        UTMLetterDesignator = "U"
    ElseIf ((48# > Lat) And (Lat >= 40#)) Then
        UTMLetterDesignator = "T"
    ElseIf ((40# > Lat) And (Lat >= 32#)) Then
        UTMLetterDesignator = "S"
    ElseIf ((32# > Lat) And (Lat >= 24#)) Then
        UTMLetterDesignator = "R"
    ElseIf ((24# > Lat) And (Lat >= 16#)) Then
        UTMLetterDesignator = "Q"
    ElseIf ((16# > Lat) And (Lat >= 8#)) Then
        UTMLetterDesignator = "P"
    ElseIf ((8# > Lat) And (Lat >= 0#)) Then
        UTMLetterDesignator = "N"
    ElseIf ((0# > Lat) And (Lat >= -8#)) Then
        UTMLetterDesignator = "M"
    ElseIf ((-8# > Lat) And (Lat >= -16#)) Then
        UTMLetterDesignator = "L"
    ElseIf ((-16# > Lat) And (Lat >= -24#)) Then
        UTMLetterDesignator = "K"
    ElseIf ((-24# > Lat) And (Lat >= -32#)) Then
        UTMLetterDesignator = "J"
    ElseIf ((-32# > Lat) And (Lat >= -40#)) Then
        UTMLetterDesignator = "H"
    ElseIf ((-40# > Lat) And (Lat >= -48#)) Then
        UTMLetterDesignator = "G"
    ElseIf ((-48# > Lat) And (Lat >= -56#)) Then
        UTMLetterDesignator = "F"
    ElseIf ((-56# > Lat) And (Lat >= -64#)) Then
        UTMLetterDesignator = "E"
    ElseIf ((-64# > Lat) And (Lat >= -72#)) Then
        UTMLetterDesignator = "D"
    ElseIf ((-72# > Lat) And (Lat >= -80#)) Then
        UTMLetterDesignator = "C"
    Else
        UTMLetterDesignator = "Z"   ' This is here as an error flag to show
                                    ' that the Latitude is outside the UTM limits.
    End If
'
'
'
End Function

Public Function ListaRefEllipsoid() As String()
'
'   Ritorna i nomi degli Ellissoidi di riferimento
'   disponibili:
'
    Dim I&
'
    ' Prepara, se e' il caso, la tavola EllipsoidT():
    If EllipsoidT(1).Name = "" Then Ellipsoid
    ReDim EllipsoidN$(1 To UBound(EllipsoidT))
'
    For I = 1 To UBound(EllipsoidN$)
        EllipsoidN$(I) = EllipsoidT(I).Name
    Next I
'
    ListaRefEllipsoid = EllipsoidN$()
'
'
'
End Function


Public Function IsUTMZone(ByVal UTMZone$) As Boolean
'
'   Verifica che una zona UTM sia scritta in modo corretto.
'   (Ver. Volpian2.1)
'
    Dim ZN&, ZL$
'
    On Error GoTo IsUTMZone_ERR
'
    ZN = Val(UTMZone$)
    ZL$ = UCase$(Trim$(Mid$(UTMZone$, InStr(UTMZone$, " "))))
'
IsUTMZone_ERR:
    IsUTMZone = (1 <= ZN And ZN <= 60) And ("C" <= ZL$ And ZL$ <= "X") And (Err = 0)
'
'
'
End Function
