Attribute VB_Name = "modUtilitaLL"
'================================================================
' Descrizione.....: Utilita' di verifica e conversione dei vari
'                   formati angolari di Longitudine e Latitudine.
' Nome dei Files..: modUtilitaLL.bas
' Data............: 29/8/2000 (Revisione)
' Versione........: 1.0 a 32 bits.
'                   (Volpian 2.1 27/7/1999)
' Sistema.........: VB6 sotto Windows NT.
' Scritto da......: F. Languasco 
' E-Mail..........: MC7061@mclink.it
'================================================================
'
'   Comprende:
'    IsLatGms_p:    Verifica che Latitudini e Longitudini,
'    IsLngGms_p:    espresse nel formato Gms_p, abbiano
'                   valori validi.
'
'    Conversioni DaToA:
'       Da:|
'    A:    | Gms_p |  Gms  |  Grd  |  Mm  |  Rad  |
'   -------|-------|-------|-------|------|-------|
'    Gms_p |   -   |   1   |   2   |      |       |
'   -------|-------|-------|-------|------|-------|
'      Gms |   1   |   -   |   1   |  1*  |   2   |
'   -------|-------|-------|-------|------|-------|
'      Grd |   2   |   1   |   -   |  1*  |   1   |
'   -------|-------|-------|-------|------|-------|
'       Mm |       |   1*  |   1*  |  -   |       |
'   -------|-------|-------|-------|------|-------|
'      Rad |       |   1   |   1   |      |   -   |
'   ----------------------------------------------*
'    1: conversione diretta;  2: conversione derivata.
'    *: due conversioni diverse per Latitudine e Longitudine.
'
'    Gms_p$: Gradi.Minuti.Secondi    [[E|W]|[N|S]|[+|-]][g][g]g.mm.ss
'    Gms$:   GradiMinutiSecondi      [[E|W]|[N|S]|[+|-]][g][g][g][m][m][s]s
'    Grd!:   Gradi decimali          [+|-]g.dddddd
'    Mm&:    Mercator meters
'    Rad!:   Radianti
'
'    FormatLatGms_p$:   converte da Gradi.Minuti.Secondi
'    FormatLngGms_p$:   a  Gradi.Minuti'.Secondi''
'
Option Explicit
'
Private Const PI# = 3.14159265358979         ' 4# * Atn(1#)
Private Const GRD_TO_RAD# = PI / 180#
Private Const RAD_TO_GRD# = 180# / PI
'
Private Const R_TERRA# = 180# * 60# / PI     ' Raggio della Terra
                                             ' in miglia nautiche.
'
Private Const NM_TO_M# = 1849.104            ' Fattore di conversione da
                                             ' miglia nautiche a metri.
                                             ' (Valore usato dal GPS)
'
Private Const R_TERRA_M# = R_TERRA * NM_TO_M ' Raggio Terra in [m].
'
Private Const S_p$ = "."                     ' Separatore in Gms_p$.
Public Function GrdToRad(ByVal LLGrd As Single) As Single
'
'   Converte gli Angoli da Gradi Decimali [Grd] a Radianti [Rad].
'   (Ver. Volpian2.1)
'
    GrdToRad = LLGrd * GRD_TO_RAD
'
'
'
End Function

Public Function LatGrdToMm(ByVal LatGrd As Single) As Long
'
'   Converte la Latitudine da Gradi Decimali [Grd]
'   a Mercator meters [Mm].
'   (Ver. Volpian2.1)
'
    Dim S#
'
    S = PI / 4# + CDbl(LatGrd) * GRD_TO_RAD / 2#
    If S >= PI / 2# Then S = (PI / 2#) - 0.000001
    S = Log(Tan(S))
    LatGrdToMm = CLng(S * R_TERRA_M)
'
'
'
End Function
Public Function LatGmsToMm(ByVal LatGms$) As Long
'
'   Converte la Latitudine da GradiMinutiSecondi [Gms]
'   a Mercator meters [Mm].
'   (Ver. Volpian2.1)
'
    Dim S#
'
    S = PI / 4# + CDbl(GmsToRad(LatGms$)) / 2#
    If S >= PI / 2# Then S = (PI / 2#) - 0.000001
    S = Log(Tan(S))
    LatGmsToMm = CLng(S * R_TERRA_M)
'
'
'
End Function

Public Function GmsToRad(ByVal LLGms$) As Single
'
'   Converte gli Angoli da GradiMinutiSecondi [Gms] a Radianti [Rad].
'   (Ver. Volpian2.1)
'
    Dim Prf$, SW As Boolean, LL_G As Single, LL_M As Single, LL_S As Single
'
    Prf$ = UCase$(Left$(LLGms$, 1))
    If Prf$ = "W" Or Prf$ = "S" Or Prf$ = "-" Then
        LLGms$ = Mid$(LLGms$, 2)
        SW = True
    ElseIf Prf$ = "E" Or Prf$ = "N" Or Prf$ = "+" Then
        LLGms$ = Mid$(LLGms$, 2)
        SW = False
    Else
        SW = False
    End If
    LLGms$ = Right$("0000000" & LLGms$, 7)
'
    LL_G = CSng(Mid$(LLGms$, 1, 3))    ' Gradi
    LL_M = CSng(Mid$(LLGms$, 4, 2))    ' Minuti
    LL_S = CSng(Mid$(LLGms$, 6, 2))    ' Secondi
'
    GmsToRad = (LL_G + LL_M / 60! + LL_S / 3600!) * GRD_TO_RAD
    If SW Then GmsToRad = -GmsToRad
'
'
'
End Function

Public Function LatMmToGms$(ByVal LatMm As Long)
'
'   Converte la Latitudine da Mercator meters [Mm]
'   a GradiMinutiSecondi [Gms].
'   (Ver. Volpian2.1)
'
    Dim S#
'
    S = CDbl(LatMm) / R_TERRA_M
    LatMmToGms$ = RadToGms$(CSng(2# * (Atn(Exp(S)) - PI / 4#)))
'
'
'
End Function
Public Function RadToGms$(ByVal LLRad As Single)
'
'   Converte gli Angoli da Radianti [Rad] a GradiMinutiSecondi [Gms].
'   (Ver. Volpian2.1)
'
    RadToGms$ = GrdToGms$(LLRad * RAD_TO_GRD)
'
'
'
End Function
Public Function RadToGrd(ByVal LLRad As Single) As Single
'
'   Converte gli Angoli da Radianti [Rad] a Gradi Decimali [Grd].
'   (Ver. Volpian2.1)
'
    RadToGrd = LLRad * RAD_TO_GRD
'
'
'
End Function

Public Function LatMmToGrd(ByVal LatMm As Long) As Single
'
'   Converte la Latitudine da Mercator meters [Mm]
'   a Gradi decimali [Grd].
'   (Ver. Volpian2.1)
'
    Dim S#
'
    S = CDbl(LatMm) / R_TERRA_M
    LatMmToGrd = CSng((2# * (Atn(Exp(S)) - PI / 4#)) * RAD_TO_GRD)
'
'
'
End Function
Public Function LngGrdToMm(ByVal LngGrd As Single) As Long
'
'   Converte la Longitudine da Gradi Decimali [Grd]
'   a Mercator meters [Mm].
'   (Ver. Volpian2.1)
'
    LngGrdToMm = CLng(LngGrd * 60! * NM_TO_M)
'
'
'
End Function
Public Function LngGmsToMm(ByVal LatGms$) As Long
'
'   Converte la Longitudine da GradiMinutiSecondi [Gms]
'   a Mercator meters [Mm].
'   (Ver. Volpian2.1)
'
    LngGmsToMm = CLng(GmsToGrd(LatGms$) * 60! * NM_TO_M)
'
'
'
End Function

Public Function LngMmToGms$(ByVal LngMm As Long)
'
'   Converte la Longitudine da Mercator meters [Mm]
'   a GradiMinutiSecondi [Gms].
'   (Ver. Volpian2.1)
'
    LngMmToGms$ = GrdToGms$(CSng(LngMm) / (60! * NM_TO_M))
'
'
'
End Function
Public Function LngMmToGrd(ByVal LngMm As Long) As Single
'
'   Converte la Longitudine da Mercator meters [Mm]
'   a Gradi decimali [Grd].
'   (Ver. Volpian2.1)
'
    LngMmToGrd = CSng(LngMm) / (60! * NM_TO_M)
'
'
'
End Function

Public Function GmsToGrd(ByVal LLGms$) As Single
'
'   Converte gli Angoli da GradiMinutiSecondi [Gms]
'   a Gradi Decimali [Grd].
'   (Ver. Volpian2.1)
'
    Dim Prf$, SW As Boolean, LL_G As Single, LL_M As Single, LL_S As Single
'
    Prf$ = UCase$(Left$(LLGms$, 1))
    If Prf$ = "W" Or Prf$ = "S" Or Prf$ = "-" Then
        LLGms$ = Mid$(LLGms$, 2)
        SW = True
    ElseIf Prf$ = "E" Or Prf$ = "N" Or Prf$ = "+" Then
        LLGms$ = Mid$(LLGms$, 2)
        SW = False
    Else
        SW = False
    End If
    LLGms$ = Right$("0000000" & LLGms$, 7)
'
    LL_G = CSng(Mid$(LLGms$, 1, 3))    ' Gradi
    LL_M = CSng(Mid$(LLGms$, 4, 2))    ' Minuti
    LL_S = CSng(Mid$(LLGms$, 6, 2))    ' Secondi
'
    GmsToGrd = LL_G + LL_M / 60! + LL_S / 3600!
    If SW Then GmsToGrd = -GmsToGrd
'
'
'
End Function
Public Function IsLatGms_p(ByVal LatGms_p$) As Boolean
'
'   Verifica che una Latitudine, in Gradi.Minuti.Secondi [Gms_p],
'   sia scritta in modo corretto.
'   (Ver. Volpian2.1)
'
    Dim Prf$, ILL As Boolean
'
    On Error GoTo IsLatGms_p_ERR
'
    Prf$ = UCase$(Left$(LatGms_p$, 1))
    If Prf$ = "S" Or Prf$ = "N" Or Prf$ = "-" Or Prf$ = "+" Then
        LatGms_p$ = Mid$(LatGms_p$, 2)
    End If
'
    ' Verifica il valore massimo:
    ILL = (Gms_pToGrd(LatGms_p$) <= 90!) And (Len(LatGms_p$) < 9)
'
    ' Verifica i secondi:
    ILL = ILL And SoloNumeri(Right$(LatGms_p$, 2)) And (CInt(Right$(LatGms_p$, 2)) < 60)
    LatGms_p$ = Left$(LatGms_p$, Len(LatGms_p$) - 2)
    ILL = ILL And (Right$(LatGms_p$, 1) = S_p$)
    LatGms_p$ = Left$(LatGms_p$, Len(LatGms_p$) - 1)
'
    ' Verifica i minuti:
    ILL = ILL And SoloNumeri(Right$(LatGms_p$, 2)) And (CInt(Right$(LatGms_p$, 2)) < 60)
    LatGms_p$ = Left$(LatGms_p$, Len(LatGms_p$) - 2)
    ILL = ILL And (Right$(LatGms_p$, 1) = S_p$)
    LatGms_p$ = Left$(LatGms_p$, Len(LatGms_p$) - 1)
'
    ' Verifica i gradi:
    ILL = ILL And SoloNumeri(LatGms_p$)
'
'
IsLatGms_p_ERR:
    IsLatGms_p = ILL And (Err = 0)
'
'
'
End Function
Public Function IsLngGms_p(ByVal LngGms_p$) As Boolean
'
'   Verifica che una Longitudine, in Gradi.Minuti.Secondi [Gms_p],
'   sia scritta in modo corretto.
'   (Ver. Volpian2.1)
'
    Dim Prf$, ILL As Boolean
'
    On Error GoTo IsLngGms_p_ERR
'
    Prf$ = UCase$(Left$(LngGms_p$, 1))
    If Prf$ = "W" Or Prf$ = "E" Or Prf$ = "-" Or Prf$ = "+" Then
        LngGms_p$ = Mid$(LngGms_p$, 2)
    End If
'
    ' Verifica il valore massimo:
    ILL = (Gms_pToGrd(LngGms_p$) <= 180!) And (Len(LngGms_p$) < 10)
'
    ' Verifica i secondi:
    ILL = ILL And SoloNumeri(Right$(LngGms_p$, 2)) And (CInt(Right$(LngGms_p$, 2)) < 60)
    LngGms_p$ = Left$(LngGms_p$, Len(LngGms_p$) - 2)
    ILL = ILL And (Right$(LngGms_p$, 1) = S_p$)
    LngGms_p$ = Left$(LngGms_p$, Len(LngGms_p$) - 1)
'
    ' Verifica i minuti:
    ILL = ILL And SoloNumeri(Right$(LngGms_p$, 2)) And (CInt(Right$(LngGms_p$, 2)) < 60)
    LngGms_p$ = Left$(LngGms_p$, Len(LngGms_p$) - 2)
    ILL = ILL And (Right$(LngGms_p$, 1) = S_p$)
    LngGms_p$ = Left$(LngGms_p$, Len(LngGms_p$) - 1)
'
    ' Verifica i gradi:
    ILL = ILL And SoloNumeri(LngGms_p$)
'
'
IsLngGms_p_ERR:
    IsLngGms_p = ILL And (Err = 0)
'
'
'
End Function
Private Function SoloNumeri(ByVal SSN$) As Boolean
'
'   Verifica che la stringa SSN$ contenga solo
'   caratteri numerici e che non sia vuota.
'   (Ver. Volpian2.1)
'
    Dim I&, C$
'
    For I = 1 To Len(SSN$)
        C$ = Mid$(SSN$, I, 1)
        If C$ < "0" Or "9" < C$ Then
            SoloNumeri = False
            Exit Function
        End If
    Next I
'
    SoloNumeri = (Len(SSN$) > 0)
'
'
'
End Function

Public Function Gms_pToGms$(ByVal LLGms_p$)
'
'   Converte da Gradi.Minuti.Secondi [Gms_p] a GradiMinutiSecondi [Gms].
'   (Ver. Volpian2.1)
'
    Dim Prf$, LLGms$
'
    Prf$ = UCase$(Left$(LLGms_p$, 1))
    If Prf$ = "W" Or Prf$ = "E" Or Prf$ = "S" Or Prf$ = "N" _
                  Or Prf$ = "-" Or Prf$ = "+" Then
        LLGms_p$ = Mid$(LLGms_p$, 2)
    Else
        Prf$ = ""
    End If
    LLGms_p$ = Right$("00" & LLGms_p$, 9)
'
    LLGms$ = Left$(LLGms_p$, 3) & Mid$(LLGms_p$, 5, 2) & Right$(LLGms_p$, 2)
    Gms_pToGms$ = Prf$ & LLGms$
'
'
'
End Function
Public Function GrdToGms_p$(ByVal LLGrd As Single)
'
'   Converte i Gradi Decimali [Grd] in Gradi.Minuti.Secondi [Gms_p].
'   (Ver. Volpian2.1)
'
    GrdToGms_p$ = GmsToGms_p(GrdToGms(LLGrd))
'
'
'
End Function
Public Function GrdToGms$(ByVal LLGrd As Single)
'
'   Converte i Gradi Decimali [Grd] in GradiMinutiSecondi [Gms].
'   (Ver. Volpian2.1)
'
    Dim SW As Boolean, LL_G&, LL_M&, LL_S&, LLGms$
'
    SW = (LLGrd < 0)
    LLGrd = Abs(LLGrd)
'
    LL_G = CLng(Int(LLGrd))
    LL_M = CLng(Int(60! * (LLGrd - CSng(LL_G))))
    LL_S = CLng(60! * (60! * (LLGrd - CSng(LL_G)) - CSng(LL_M)))
    If LL_S = 60 Then
        LL_S = 0
        LL_M = LL_M + 1
    End If
    If LL_M = 60 Then
        LL_M = 0
        LL_G = LL_G + 1
    End If
'
    LLGms$ = Format$(LL_G, "000") & Format$(LL_M, "00") & Format$(LL_S, "00")
    GrdToGms$ = IIf(SW, "-" & LLGms$, LLGms$)
'
'
'
End Function


Public Function GmsToGms_p$(ByVal LLGms$)
'
'   Converte da GradiMinutiSecondi [Gms] a Gradi.Minuti.Secondi [Gms_p].
'   (Ver. Volpian2.1)
'
    Dim Prf$, LLGms_p$
'
    Prf$ = UCase$(Left$(LLGms$, 1))
    If Prf$ = "W" Or Prf$ = "E" Or Prf$ = "S" Or Prf$ = "N" _
                  Or Prf$ = "-" Or Prf$ = "+" Then
        LLGms$ = Mid$(LLGms$, 2)
    Else
        Prf$ = ""
    End If
    LLGms$ = Right$("0000000" & LLGms$, 7)
'
    LLGms_p$ = Left$(LLGms$, 3) & S_p$ & Mid$(LLGms$, 4, 2) & S_p$ & Right$(LLGms$, 2)
    If Left$(LLGms_p$, 1) = "0" Then LLGms_p$ = Mid$(LLGms_p$, 2)
    If Left$(LLGms_p$, 1) = "0" Then LLGms_p$ = Mid$(LLGms_p$, 2)
'
    GmsToGms_p$ = Prf$ & LLGms_p$
'
'
'
End Function
Public Function Gms_pToGrd(ByVal LLGms_p$) As Single
'
'   Converte gli Angoli da Gradi.Minuti.Secondi [Gms_p]
'   a Gradi Decimali [Grd].
'   (Ver. Volpian2.1)
'
    Gms_pToGrd = GmsToGrd(Gms_pToGms(LLGms_p$))
'
'
'
End Function



Public Function FormatLatGms_p$(ByVal LatGms_p$)
'
'   Converte una Latitudine da Gradi.Minuti.Secondi [Gms_p]
'   a Gradi.Minuti'.Secondi''.
'   (Ver. Volpian2.1)
'
    Dim Prf$, Lat_G$, Lat_M$, Lat_S$
'
    Prf$ = UCase$(Left$(LatGms_p$, 1))
    If Prf$ = "S" Or Prf$ = "-" Then
        Prf$ = "S"
        LatGms_p$ = Mid$(LatGms_p$, 2)
    ElseIf Prf$ = "N" Or Prf$ = "+" Then
        Prf$ = "N"
        LatGms_p$ = Mid$(LatGms_p$, 2)
    Else
        Prf$ = "N"
    End If
'
    ' Trova i secondi:
    Lat_S$ = Right$(LatGms_p$, 2)
    LatGms_p$ = Left$(LatGms_p$, Len(LatGms_p$) - 3)
'
    ' Trova i minuti:
    Lat_M$ = Right$(LatGms_p$, 2)
    LatGms_p$ = Left$(LatGms_p$, Len(LatGms_p$) - 3)
'
    ' Trova i gradi:
    Lat_G$ = Right$("00" & LatGms_p$, 2)
'
    FormatLatGms_p$ = Prf$ & Lat_G$ & "." & Lat_M$ & "'." & Lat_S$ & "''"
'
'
'
End Function

Public Function FormatLngGms_p$(ByVal LngGms_p$)
'
'   Converte una Longitudine da Gradi.Minuti.Secondi [Gms_p]
'   a Gradi.Minuti'.Secondi''.
'   (Ver. Volpian2.1)
'
    Dim Prf$, Lng_G$, Lng_M$, Lng_S$
'
    Prf$ = UCase$(Left$(LngGms_p$, 1))
    If Prf$ = "W" Or Prf$ = "-" Then
        Prf$ = "W"
        LngGms_p$ = Mid$(LngGms_p$, 2)
    ElseIf Prf$ = "E" Or Prf$ = "+" Then
        Prf$ = "E"
        LngGms_p$ = Mid$(LngGms_p$, 2)
    Else
        Prf$ = "E"
    End If
'
    ' Trova i secondi:
    Lng_S$ = Right$(LngGms_p$, 2)
    LngGms_p$ = Left$(LngGms_p$, Len(LngGms_p$) - 3)
'
    ' Trova i minuti:
    Lng_M$ = Right$(LngGms_p$, 2)
    LngGms_p$ = Left$(LngGms_p$, Len(LngGms_p$) - 3)
'
    ' Trova i gradi:
    Lng_G$ = Right$("000" & LngGms_p$, 3)
'
    FormatLngGms_p$ = Prf$ & Lng_G$ & "." & Lng_M$ & "'." & Lng_S$ & "''"
'
'
'
End Function
